/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_3DMATH_SEGMENT_H__INCLUDED__
#define __GAUGE3D_3DMATH_SEGMENT_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_3DMATH_LINE_H__INCLUDED__
#include "line.h"
#endif //redundant include guard

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GSegment segment.h gauge3d/3dmath/segment.h
 *  \ingroup d3math
 *  \brief Represents a line segment.
 */
class GAUGE3D_API GSegment
{
	GVector mEndPoints[2];

public:
	// lifecycle -------------------------------------------------------
	//! Creates an uninitialized segment.
	GSegment();
	//! Creates a segment with the specified endpoints.
	GSegment(const GVector& endPoint1, const GVector& endPoint2);
	
	// operations ------------------------------------------------------
	//! Gets the line which the segment is on.
	void GetLine(GLine* line) const;
	
	//! Returns true if the point is on the segment.
	bool Contains(const GVector& point);
	
	// operators -------------------------------------------------------
	//! Array-like access to endpoints.
	GVector& operator[](int index);
	//! Array-like access to endpoints.
	const GVector& operator[](int index) const;
};

// inline member functions ===========================================
// lifecycle ---------------------------------------------------------
inline
GSegment::GSegment()
{
}

// operators ---------------------------------------------------------
inline GVector&
GSegment::operator[](int index)
{
	return mEndPoints[index];
}

inline const GVector&
GSegment::operator[](int index) const
{
	return mEndPoints[index];
}

} // namespace GAUGE3D

#endif //included
