/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
// component header --------------------------------------------------
#include "vector.h"

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class state definition ============================================

// class member functions ============================================
// lifecycle ---------------------------------------------------------
// operations --------------------------------------------------------

//Slerp = Spherical Linear Interpolation.  This interpolates between two
// unit vectors along a sphere.  This is great for interpolating normals.
// The result is stored in this.
void
GVector::Slerp(const GVector& v1, const GVector& v2, GCoordinate t)
{
	GCoordinate angle, dot;
	GCoordinate s1, s2;
	
	dot = v1 * v2;
	if(IsApproxZero(dot - 1.0f))
	{
		//vectors are equal or opposite
		*this = v1;
		return;
	}
	
	angle = (GCoordinate)acos(dot);
	
	s1 = (GCoordinate)(1.0f / sin(angle));
	s2 = (GCoordinate)(sin(angle * t) * s1);
	s1 = (GCoordinate)(sin(angle * (1.0f - t)) * s1);
	
	x = s1 * v1.x + s2 * v2.x;
	y = s1 * v1.y + s2 * v2.y;
	z = s1 * v1.z + s2 * v2.z;
}

// accessors ---------------------------------------------------------
// inquiry -----------------------------------------------------------
// operators ---------------------------------------------------------
bool
GVector::operator==(const GVector& other) const
{
	//correct rounding error.
	return IsApproxZero(x - other.x)
	    && IsApproxZero(y - other.y)
	    && IsApproxZero(z - other.z);
}

// free operators ====================================================

//Vector division (not supported by math!)...
//This function assumes that the two vectors are pointing in the same
// direction, then attempts to divide them.
GCoordinate
operator/(const GVector& vec1, const GVector& vec2)
{
	//make sure denom isn't 0.
	if(!IsApproxZero(vec2.x))
		return vec1.x / vec2.x;
	else if(!IsApproxZero(vec2.y))
		return vec1.y / vec2.y;
	else
		return vec1.z / vec2.z;
}

} // namespace GAUGE3D
