/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
// component header --------------------------------------------------
#include "billboardsprite.h"

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class state definition ============================================
struct GBillboardSprite::Private
{
	// state variables -------------------------------------------------
	GCoordinate mWidth;
	GCoordinate mHeight;
	GVector mTexCoords[4];
	static uint16 mIndices[6];
	
	// private functions -----------------------------------------------
};

uint16 GBillboardSprite::Private::mIndices[6] = {0, 1, 2, 0, 2, 3};

// class member functions ============================================
// lifecycle ---------------------------------------------------------
GBillboardSprite::GBillboardSprite(GCoordinate width, GCoordinate height)
	: mPrivate(*(new Private))
{
	mPrivate.mWidth  = width;
	mPrivate.mHeight = height;
	
	mPrivate.mTexCoords[0] = GVector(0,0,0);
	mPrivate.mTexCoords[1] = GVector(1,0,0);
	mPrivate.mTexCoords[2] = GVector(1,1,0);
	mPrivate.mTexCoords[3] = GVector(0,1,0);
}

GBillboardSprite::~GBillboardSprite()
{
	delete &mPrivate;
}

// access ------------------------------------------------------------
void
GBillboardSprite::SetDimentions(GCoordinate  width, GCoordinate  height)
{
	mPrivate.mWidth  = width;
	mPrivate.mHeight = height;
}

void
GBillboardSprite::GetDimentions(GCoordinate* width, GCoordinate* height)
{
	*width  = mPrivate.mWidth;
	*height = mPrivate.mHeight;
}

// operations --------------------------------------------------------
void
GBillboardSprite::TexCoords(GVector upperLeft, GVector lowerRight)
{
	mPrivate.mTexCoords[0] = upperLeft;
	mPrivate.mTexCoords[1] = GVector(lowerRight.x,upperLeft.y,0);
	mPrivate.mTexCoords[2] = lowerRight;
	mPrivate.mTexCoords[3] = GVector(upperLeft.x,lowerRight.y,0);
}

GBillboardSprite::BillboardAnimState::~BillboardAnimState()
{}

GModel::pAnimState
GBillboardSprite::CreateAnimState()
{
	return new BillboardAnimState;
}

// rendering info ----------------------------------------------------
class GBillboardSprite::BillboardRenderInfo: public GModel::RenderInfo
{
	GBillboardSprite* mModel;
	GQuaternion mViewAngle;
	
public:
	BillboardRenderInfo(GBillboardSprite* model): mModel(model) {}
	~BillboardRenderInfo() {}
	
	int StartFrame(pAnimState animState, const GVector& viewPoint,
	               const GQuaternion& viewAngle, const GFrustum& viewFrustum,
	               float geoLod)
	{
		mViewAngle = viewAngle;
		return 1;
	}
	
	//these two are defined below
	void GetStaticInfo(RenderSet* target, int setNum)
	{
		target->mDynamicFlags = RenderSet::VERTEX_POSITION;
		
		target->mNumVertices = 4;
		target->mTexCoords = mModel->mPrivate.mTexCoords;
		
		target->mNumIndices = 6;
		target->mIndexType = RenderSet::TRIANGLES;
		target->mIndices = mModel->mPrivate.mIndices;
	}
	
	void GetDynamicInfo(RenderSet* target, int setNum)
	{
		GVector basis[3] = {GVECTOR_FORWARD, GVECTOR_UP, GVECTOR_RIGHT};
		
		mViewAngle.Rotate(basis, basis, 3);
		
		GCoordinate x = mModel->mPrivate.mWidth * 0.5;
		GCoordinate y = mModel->mPrivate.mHeight * 0.5;
		
		target->mNumVertices = 4;
		target->mVertices[0] = -x * basis[2] + y * basis[1];
		target->mVertices[1] =  x * basis[2] + y * basis[1];
		target->mVertices[2] =  x * basis[2] - y * basis[1];
		target->mVertices[3] = -x * basis[2] - y * basis[1];
		
		basis[0] *= -1.0f;
		
		target->mNormals[0] = basis[0];
		target->mNormals[1] = basis[0];
		target->mNormals[2] = basis[0];
		target->mNormals[3] = basis[0];
	}
};

GModel::pRenderInfo
GBillboardSprite::CreateRenderInfo()
{
	return new BillboardRenderInfo(this);
}


GAUGE3D_DECLARE_INTERFACES(GBillboardSprite)
	GAUGE3D_DECLARE_BASECLASS(GModel)
GAUGE3D_END_INTERFACES

GAUGE3D_DECLARE_INTERFACES(GBillboardSprite::BillboardAnimState)
	GAUGE3D_DECLARE_BASECLASS(AnimState)
GAUGE3D_END_INTERFACES

} //namespace GAUGE3D
