/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_GRAPHICS_SPHEREMODEL_H__INCLUDED__
#define __GAUGE3D_GRAPHICS_SPHEREMODEL_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_GRAPHICS_MODEL_H__INCLUDED__
#include "model.h"
#endif //redundant include guard

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GSphereModel;
/*! \typedef pGSphereModel
 *  \ingroup graphics
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GSphereModel GSphereModel\endlink. */
typedef GSmartPointer<GSphereModel> pGSphereModel;

/*! \class GSphereModel spheremodel.h gauge3d/graphics/spheremodel.h
 * \ingroup graphics
 * \brief A sphere model.
 *
 * GSphereModel is a derivative of GModel.  You can use it to display spheres.
 * The sphere is of the geodesic type, which makes it look pretty.
 *
 * Note that you cannot set the radius of the sphere; it is always 1.0.
 * You can, however, give GSprite a scale factor, which will cause the
 * sphere to be drawn with a radius equal to that factor. */
class GAUGE3D_API GSphereModel: public GModel
{
	struct Private;
	Private& mPrivate;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781eb7);
public:
	// lifecycle -------------------------------------------------------
	GSphereModel();
	virtual ~GSphereModel();
	
	// operations -----------------------------------------------------
	/*! \class SphereModelAnimState spheremodel.h gauge3d/graphics/spheremodel.h
	 *  \brief AnimState for a GSphereModel.
	 *
	 *  There is absolutely nothing special about this class.  It doesn't
	 *  actually do anything at all.  Spheres are not animated.  They are
	 *  static models. */
	class GAUGE3D_API SphereModelAnimState: public GModel::AnimState
	{
		SphereModelAnimState();
		friend class GSphereModel;
		
		GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781ec8);
	public:
		virtual ~SphereModelAnimState();
		
		//Static model.  No animation state.
	};
	typedef GSmartPointer<SphereModelAnimState> pSphereModelAnimState;
	
	friend class SphereModelAnimState;
	
	//CreateAnimState() generates an AnimState object associated with this model.
	virtual pAnimState CreateAnimState();
	
	// rendering info ----------------------------------------------
	pRenderInfo CreateRenderInfo();
};

} // namespace GAUGE3D

#endif //included
