/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_RENDERER_CAMERA_H__INCLUDED__
#define __GAUGE3D_RENDERER_CAMERA_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
#ifndef __GAUGE3D_3DMATH_VECTOR_H__INCLUDED__
#include "gauge3d/3dmath/vector.h"
#endif

#ifndef __GAUGE3D_3DMATH_QUATERNION_H__INCLUDED__
#include "gauge3d/3dmath/quaternion.h"
#endif

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GCamera camera.h gauge3d/renderer/camera.h
 *  \ingroup renderer
 *  \brief Represents the viewpoint into the 3D scene. */
class GAUGE3D_API GCamera: public GObject
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x399aa774);
public:
	// lifecycle -------------------------------------------------------
	virtual ~GCamera();
	
	// operations ------------------------------------------------------
	//! Sets the position of the camera in the scene.
	virtual void Position(const GVector& position) = 0;
	//! Sets the direction in which the camera is facing.
	virtual void Direction(const GQuaternion& direction) = 0;
	/*! \brief Sets the magnification of the camera.
	 *
	 * Zoom!  1.0 is normal, higher is zoomed in.  Setting the zoom below
	 * 1.0 is probably a bad idea, as this will increase the FOV beyond
	 * 90 degrees, which could cause distortion. */
	virtual void Zoom(GCoordinate zoom) = 0;
};
/*! \typedef pGCamera
 *  \ingroup renderer
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GCamera GCamera\endlink. */
typedef GSmartPointer<GCamera> pGCamera;

} //namespace GAUGE3D

#endif //included
