/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

#ifndef GAUGE3D_STDDEFS_H__INCLUDED_
#define GAUGE3D_STDDEFS_H__INCLUDED_


namespace GAUGE3D {

//Config header
#ifdef LIBGAUGE3D
#include "config.h"
#include "platform.h"
#else
#include "platform.h"
#endif //LIBGAUGE3D

#ifdef PLATFORM_MSVC
//MSVC is not as smart as it seems to think it is.
//Basically, MSVC assumes you are stupid and warns you on what it
// thinks are mistakes.  Usually, these "mistakes" are entirely
// intentional, and I actually want it to do exactly what I wrote!

//C4786 tells you when a symbol is too large for the debugger.
//This happens basically whenever you use an STL class.
//Disable!
#pragma warning( disable : 4786 )

//C4503 is basically the same, except it is not the debugger's
// fault.  It is the compiler.
#pragma warning( disable : 4503 )

//C4251 - MSVC, as a compiler, you should know that template
// classes do not need to be exported by the DLL.  As a matter
// of fact, you should also know that they CAN'T be exported
// anyway!
#pragma warning( disable : 4251 )

//C4305 - 'identifier' : truncation from 'type' to 'type'
// Look, MSVC, I am not an idiot.  I know how big my vairables are.
#pragma warning( disable : 4305 )
#pragma warning( disable : 4244 )

//C4800 - Forcing integer value to boolean "true" or "false".
// Um...  yeah...  that's exactly what I told you to do.
#pragma warning( disable : 4800 )

//C4291 - No coresponding "delete" for "new" blah blah exceptions blah.
// Just shut up, damnit.
#pragma warning( disable : 4291 )

//C4284 - MSVC: "Warning!  This smart pointer points to an integer, so
// operator-> won't work!"  Temporal:  "No shit Sherlock."
#pragma warning( disable : 4284 )

#endif

#ifdef PLATFORM_MSVC
# ifdef GAUGE3D_EXPORTS
#  define GAUGE3D_API __declspec(dllexport)
# else
#  define GAUGE3D_API __declspec(dllimport)
# endif
#else
# define GAUGE3D_API
#endif

//Invalid pointer
#ifndef NULL
#define NULL 0
#endif

//Unix math.h defines M_PI, Windows doesn't define anything.
//To avoid confusion, we define everything!
/*! \name PI definitions
 *  \brief To avoid ambiguities in pi constants, we define both PI and M_PI. */
//@{
/*! \ingroup common
 *  \brief pi */
#define M_PI    3.14159265358979323846
/*! \ingroup common
 *  \brief pi/2 */
#define M_PI_2  1.57079632679489661923
/*! \ingroup common
 *  \brief pi/4 */
#define M_PI_4  0.78539816339744830962
/*! \ingroup common
 *  \brief pi */
#define PI      3.14159265358979323846
/*! \ingroup common
 *  \brief pi/2 */
#define PI_2    1.57079632679489661923
/*! \ingroup common
 *  \brief pi/4 */
#define PI_4    0.78539816339744830962
//@}

//Standard types. ----------------------------------------
/*! \name Standard types
 *  \brief Some of these will be different on some systems. */
//@{
/*! \typedef uint
 *  \ingroup common
 *  \brief unsigned int */
typedef unsigned int           uint;
/*! \typedef ushort
 *  \ingroup common
 *  \brief unsigned short */
typedef unsigned short         ushort;
/*! \typedef ulong
 *  \ingroup common
 *  \brief unsigned long */
typedef unsigned long          ulong;

//These data types are provided because the standard types vary across
// platforms.  For example, long is 64-bit on 64-bit systems, but 32-bit
// on 32-bit systems.  These types only need to be used when doing binary
// file or network I/O.

#if defined(PLATFORM_32BIT) || defined(DOXYGEN)
/*! \typedef uchar
 *  \ingroup common
 *  \brief unsigned char */
typedef unsigned char          uchar;

/*! \typedef int8
 *  \ingroup common
 *  \brief 8-bit signed integer */
typedef char                   int8;
/*! \typedef int16
 *  \ingroup common
 *  \brief 16-bit signed integer */
typedef short                  int16;
/*! \typedef int32
 *  \ingroup common
 *  \brief 32-bit signed integer */
typedef int                    int32;
/*! \typedef int64
 *  \ingroup common
 *  \brief 64-bit signed integer */
#ifdef PLATFORM_MSVC
typedef __int64                int64;
#else
typedef long long              int64;
#endif

/*! \typedef uint8
 *  \ingroup common
 *  \brief 8-bit unsigned integer */
typedef unsigned char          uint8;
/*! \typedef uint16
 *  \ingroup common
 *  \brief 16-bit unsigned integer */
typedef unsigned short         uint16;
/*! \typedef uint32
 *  \ingroup common
 *  \brief 32-bit unsigned integer */
typedef unsigned int           uint32;
/*! \typedef uint64
 *  \ingroup common
 *  \brief 64-bit unsigned integer */
#ifdef PLATFORM_MSVC
typedef unsigned __int64       uint64;
#else
typedef unsigned long long     uint64;
#endif

/*! \typedef float32
 *  \ingroup common
 *  \brief 32-bit floating point number */
typedef float                  float32;
/*! \typedef float64
 *  \ingroup common
 *  \brief 64-bit floating point number */
typedef double                 float64;

#elif defined(PLATFORM_64BIT)
#error "FIXME: Typedefs for 64bit processors"
// FIXME: Are these correct?  I don't have a 64-bit processor, so...
typedef unsigned char          uchar;

typedef char                   int8;
typedef short short            int16;
typedef short                  int32;
typedef int                    int64;

typedef unsigned char          uint8;
typedef unsigned short short   uint16;
typedef unsigned short         uint32;
typedef unsigned int           uint64;

typedef float                  float32;
typedef double                 float64;

#else
#error "CPU word size not supported (or the configure script screwed up)"
#endif

// Other standard typedefs
/*! \typedef tTime
 *  \ingroup common
 *  \brief Time value, measured in seconds.
 *
 *  The time from which this value mesures is completely arbitrary, but will
 *  be the same for all tTime values during a particular run of the engine.
 *  If you want to get the date, use the ANSI-C functions created for this purpose. */
typedef double tTime;
//@}

} // namespace GAUGE3D

#include "gauge3d/common/memory.h"
#include "gauge3d/common/object.h"

#endif // included
