/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef __GAUGE3D_FILES_DISKDIRECTORY_H__INCLUDED__
#define __GAUGE3D_FILES_DISKDIRECTORY_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include <gauge3d/stddefs.h>
#endif //redundant include guard

// system ------------------------------------------------------------
// external packages -------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_FILES_DIRECTORY_H__INCLUDED__
#include "directory.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
/*! \class GDiskDirectory diskdirectory.h gauge3d/files/diskdirectory.h
 *  \ingroup files
 *  \brief A directory on a disk.
 *
 *  GDiskDirectory is an actual file on the hard drive or other disk. */
class GAUGE3D_API GDiskDirectory: public GDirectory
{
	// insulated state -------------------------------------------------
	struct Private;
	Private& mPrivate;
	
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39781f58);
public:
	// lifecycle -------------------------------------------------------
	//! Create without opening a directory.
	GDiskDirectory();
	//! Create and open the given directory.
	GDiskDirectory(GString dirname);
	//! Close the directory.
	virtual ~GDiskDirectory();
	
	/*! \brief Opens a particular directory.
	 *  \param dirname The path and name of the directory on disk.
	 *  \returns       True on success, false on error. */
	bool Open(GString dirname);
	//! Closes the directory.
	void Close();
	
	// inquiry ---------------------------------------------------------
	//! Returns true if the directory is open.
	bool IsOpen();
	
	// operations ------------------------------------------------------
	virtual GArray<GString> ListFiles();
	virtual GArray<GString> ListSubdirs();
	
	//Use these to open the contents of a directory.  This way, if you have
	// a pointer to the root directory of any filesystem, you can access
	// everything in the system.
	//Note that it is perfectly legal to say:
	//   myDirectory.OpenFile("this/is/a/deeply/nested/file");
	// Likewise with OpenSubdir().  You need not manually traverse the
	// directory tree.
	virtual pGFile      OpenFile(GString name);
	virtual pGDirectory OpenSubdir(GString name);
	
	//AddFile returns NULL if the file already exists
	virtual pGFile      AddFile(GString name);
	//AddSubdir returns NULL if the directory already exists
	virtual pGDirectory AddSubdir(GString name);
	
	//these two remove files and sub directories from the directory.
	virtual void RemoveFile(GString name);
	virtual void RemoveDirectory(GString name);
};
/*! \typedef pGDiskDirectory
 *  \ingroup files
 *  \brief \link GAUGE3D::GSmartPointer Smart pointer\endlink to a
 *         \link GAUGE3D::GDiskDirectory GDiskDirectory\endlink. */
typedef GSmartPointer<GDiskDirectory> pGDiskDirectory;

} //namespace GAUGE3D

#endif //included
