/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2001 Will Newton
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: pngplugin.cpp
 *      Package: Plugin
 *    Component: PngImg
 *  Description: PNG Image Plugin
 * Dependencies: GPlugin
 */

// includes ==========================================================
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_PLUGIN_H__INCLUDED__
#include "gauge3d/plugins/plugin.h"
#endif

// system ------------------------------------------------------------
// extenal packages -------------------------------------------------
// local package -----------------------------------------------------
#include "pngimage.h"

// begin namespace ===================================================
namespace GAUGE3D {

// class definition ==================================================
class GpPngPlugin: public GPlugin
{
	GAUGE3D_SET_UID(0x00000040, 0x05a64f2f, 0x3a78f371); 
	
public:
	// lifecycle -------------------------------------------------------
	GpPngPlugin() {};
	virtual ~GpPngPlugin() {};
	
	// inquiry ---------------------------------------------------------
	//these retun the human-readable name and description of the plugin.
	virtual GString Name() {return "PNG Image";}
	virtual GString Description()
		{return "Loads PNG format images.";}
	
	//InUse retuns true if the plugin is being used.  Otherwise, false.
	// The object loader uses this to determine when it is safe to unload
	// a plugin from RAM.
	virtual bool InUse() {return true;}  //FIXME
	
	// operations ------------------------------------------------------
	virtual pGObject CreateObject(const GString& className)
	{
		return NULL;   //we don't serve any non-datafile plugins.
	}
	
	//When loading a file, if the plugin finds that it can't load the thing,
	// it should retun NULL.
	virtual pGObject CreateObject(const GString& className, pGFile file)
	{
		GpPngImage *ret = new GpPngImage(file);
		if(!ret->ReadOk()) {
			delete ret;
			return NULL;
		}
		else {
			return ret;
		}
	}
	
	//The plugin tells the object loader what objects are available from
	// it by giving it an array of ObjectInfo objects.  Each one gives
	// info for one of the available objects.
	//ObjectList() retuns ObjectInfos for every object available from
	// this plugin.
	virtual GArray<ObjectInfo> ObjectList()
	{
//		GArray<ObjectInfo> info(4);
		GArray<ObjectInfo> info(1);
		
		info[0].mClassname      = "GpPngImage";
		info[0].mName           = "PNG Image";
		info[0].mDescription    = "Loads PNG images.";
		info[0].mBaseClasses    = GArray<GString>(1);
		info[0].mBaseClasses[0] = "GImage";
		info[0].mFileTypes      = GArray<GString>(1);
		info[0].mFileTypes[0]   = "png";

		return info;
	}
};

GAUGE3D_DECLARE_INTERFACES(GpPngPlugin)
	GAUGE3D_DECLARE_BASECLASS(GPlugin)
GAUGE3D_END_INTERFACES

GAUGE3D_EXPORT_PLUGIN(GpPngPlugin)

} //namespace GAUGE3D
