/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Direct3D 7 Rendering Plugin
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7display.h
 *      Package: Direct3D 7 Rendering Plugin
 *    Component: Display
 *  Description: Represents the rendering window.
 * Dependencies: GpD3D7Scene, GpD3D7Sprite, GpD3D7Light
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7DISPLAY_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_DIRECT3D7_D3D7DISPLAY_H__INCLUDED__

#ifndef DIRECTINPUT_VERSION
#define DIRECTINPUT_VERSION 0x0700
#define DIRECT3D_VERSION    0x0700
#define DIRECTDRAW_VERSION  0x0700
#endif

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#define __GAUGE3D_STDDEFS_H__INCLUDED__
#endif //redundant include guard

// system ------------------------------------------------------------
#ifndef __SYSTEM_DDRAW_H__INCLUDED__
#include <ddraw.h>
#define __SYSTEM_DDRAW_H__INCLUDED__
#endif //redundant include guard

#ifndef __SYSTEM_D3D_H__INCLUDED__
#define D3D_OVERLOADS
#include <d3d.h>
#define __SYSTEM_D3D_H__INCLUDED__
#endif //redundant include guard

#ifndef __SYSTEM_DINPUT_H__INCLUDED__
#define UNICODE
#include <dinput.h>
#undef UNICODE
#define __SYSTEM_DINPUT_H__INCLUDED__
#endif //redundant include guard

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_DISPLAY_H__INCLUDED__
#include "gauge3d/renderer/display.h"
#define __GAUGE3D_RENDERER_DISPLAY_H__INCLUDED__
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __LOCAL_DXERROR_H__INCLUDED__
#include "dxerror.h"
#define __LOCAL_DXERROR_H__INCLUDED__
#endif

#include "d3d7camera.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpD3D7Display: public GDisplay
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39782263)
public:
	// lifecycle -------------------------------------------------------
	GpD3D7Display(IDirectDraw7 *DD);
	virtual ~GpD3D7Display();
	
	// operations ------------------------------------------------------
	// operations ------------------------------------------------------
	//Open resets the display resolution and color depth.
	//This function must be called once before any rendering can be done.
	//If the display could not be openned, false is returned.
	//Passing zero for colorDepth will cause the current display depth to be used.
	virtual bool Open(int width, int height, int depth, bool fullScreen);
	
	//I think this is self-explanitory...
	virtual GArray<DisplayProps> ListAvailableDisplays();
	
	virtual void Camera(pGCamera camera);

	virtual void DoFrame(tTime time, tTime frametime);

	virtual void CaptureMouse(bool enable);

private:
	GError *mError;

	LPDIRECTDRAW7 mDD;
	LPDIRECTDRAWSURFACE7 mPrimarySurface;
	LPDIRECTDRAWSURFACE7 mBackSurface;
	LPDIRECTDRAWSURFACE7 mZBuffer;
	LPDIRECT3DDEVICE7 mDevice;

	LPDIRECTINPUT7 mDInput;
	LPDIRECTINPUTDEVICE7 mDIMouse;
	LPDIRECTINPUTDEVICE7 mDIKeyboard;

	HWND mWindow;
	bool mFullScreen;
	bool mQuitting;
	float mGeoLod;
	pGpD3D7Camera mCamera;

	static HINSTANCE mInstance;
	bool mInit;

	RECT mPosition;
	SIZE mSize;
	
	bool ProcessEvents();
	void ProcessInputs();

	HRESULT SetMouseAcquire(bool set);

	bool CreateFullScreen();
	bool CreateBackSurface();
	bool CreateDevice();
	bool CreateZBuffer();
	void SetViewport();

public:
	//Allows the externally defined window procedure to provide the display class with
	//window messages.
	LRESULT WindowProc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam);
	LPDIRECT3DDEVICE7 GetDevice(void) { return mDevice; }
};

} // namespace Plugins
} // namespace GAUGE3D

#endif //included
