/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Direct3D 7 Rendering Plugin
 * Copyright (C) 2000 Steven Haworth
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: d3d7renderer.cpp
 *      Package: Direct3D 7 Rendering Plugin
 *    Component: Renderer
 *  Description: serves rendering classes
 * Dependencies: GpD3D7*
 */

// includes ==========================================================
// component header --------------------------------------------------
#include "d3d7renderer.h"

// system ------------------------------------------------------------
#ifndef __SYSTEM_D3DX_H__INCLUDED__
#include <d3dx.h>
#define __SYSTEM_D3DX_H__INCLUDED__
#endif //redundant include guard

#include <iostream>


// external packages -------------------------------------------------
// local package -----------------------------------------------------
#include "d3d7display.h"
#include "d3d7scene.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class state definition ============================================

// class member functions ============================================
// lifecycle ---------------------------------------------------------
GpD3D7Renderer::GpD3D7Renderer()
{
	HRESULT hr;

	mError = NULL;
	mIDirectDraw = NULL;
	hr = DirectDrawCreateEx(NULL,reinterpret_cast<void **>(&mIDirectDraw),IID_IDirectDraw7,NULL);
	if(FAILED(hr)) {
		CREATE_DIRECTX_ERROR(mError,hr,"Creating IDirectDraw7 interface");
		return;
	}
}

GpD3D7Renderer::~GpD3D7Renderer()
{
	if(mIDirectDraw) {
		mIDirectDraw->Release();
		mIDirectDraw = NULL;
	}
	RELEASE_ERROR(mError);
}

// operations --------------------------------------------------------
pGDisplay
GpD3D7Renderer::NewDisplay()
{
	mDisplay = new GpD3D7Display(mIDirectDraw);
	return mDisplay;
}

pGTexture
GpD3D7Renderer::NewTexture()
{
	return new GpD3D7Texture(mIDirectDraw,mDisplay->GetDevice());
}

pGScene
GpD3D7Renderer::NewScene()
{
	return new GpD3D7Scene(mDisplay->GetDevice());
}

GAUGE3D_DECLARE_INTERFACES(GpD3D7Renderer)
	GAUGE3D_DECLARE_BASECLASS(GRenderer)
GAUGE3D_END_INTERFACES

} //namespace Plugins
} //namespace GAUGE3D
