/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: glcamera.h
 *      Package: OpenGL Rendering Plugin
 *    Component: Display
 *  Description: Represents the rendering window.
 * Dependencies: GpGLScene, GpGLSprite, GpGLLight
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLCAMERA_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLCAMERA_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_CAMERA_H__INCLUDED__
#include "gauge3d/renderer/camera.h"
#endif //redundant include guard

// local package -----------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLSCENE_H__INCLUDED__
#include "glscene.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpGLCamera: public GCamera
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x399ac1c2)
public:
	// lifecycle -------------------------------------------------------
	GpGLCamera(pGpGLScene scene);
	virtual ~GpGLCamera();
	
	// operations ------------------------------------------------------
	//Position sets the position of the camera.
	virtual void Position(const GVector& position);
	//Direction sets the direction in which the camera is facing.
	virtual void Direction(const GQuaternion& direction);
	//Zoom!  1.0 is normal, higher is zoomed in.  Setting the zoom below
	// 1.0 is probably a bad idea, as this will increase the FOV beyond
	// 90 degrees, which could cause distortion.
	virtual void Zoom(GCoordinate zoom);
	
	//Render the scene from the point of view of the camera.
	void Draw(float geoLod);
	
private:
	pGpGLScene mScene;
	GVector mPosition;
	GQuaternion mDirection;
	GCoordinate mZoom;
};
typedef GSmartPointer<GpGLCamera> pGpGLCamera;

} // namespace Plugins
} //namespace GAUGE3D

#endif //included
