/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: gldisplay.h
 *      Package: OpenGL Rendering Plugin
 *    Component: Display
 *  Description: Represents the rendering window.
 * Dependencies: GpGLScene, GpGLSprite, GpGLLight
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLDISPLAY_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLDISPLAY_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
#ifndef __SYSTEM_SDL_H__INCLUDED__
#include <SDL.h>
#define __SYSTEM_SDL_H__INCLUDED__
#endif //redundant include guard

// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_DISPLAY_H__INCLUDED__
#include "gauge3d/renderer/display.h"
#endif //redundant include guard

// local package -----------------------------------------------------
#include "glscene.h"
#include "glcamera.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpGLDisplay: public GDisplay
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x39782201)
public:
	// lifecycle -------------------------------------------------------
	GpGLDisplay();
	virtual ~GpGLDisplay();
	
	// operations ------------------------------------------------------
	// operations ------------------------------------------------------
	//Open resets the display resolution and color depth.
	//This function must be called once before any rendering can be done.
	//If the display could not be openned, false is returned.
	//Passing zero for colorDepth will cause the current display depth to be used.
	virtual bool Open(int width, int height, int depth, bool fullScreen);
	
	//I think this is self-explanitory...
	virtual GArray<DisplayProps> ListAvailableDisplays();
	
	virtual void Camera(pGCamera camera);
	
	virtual void CaptureMouse(bool enable);
	
	virtual void DoFrame(tTime time, tTime frametime);
	
private:
	static bool mSDLOpen;
	SDL_Surface* mFrameBuffer;
	int mWidth;
	int mHeight;
	float mGeoLod;   //geometric level of detail
	bool mCaptureMouse;
	pGpGLCamera mCamera;
	
	void SaveScreen();
	void ProcessEvents();
};

} // namespace Plugins
} // namespace GAUGE3D

#endif //included
