/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: gllight.h
 *      Package: OpenGL Rendering Plugin
 *    Component: Light
 *  Description: GL implementation of a light
 * Dependencies: who cares?
 */

#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLLIGHT_H__INCLUDED__
#define __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLLIGHT_H__INCLUDED__

// includes ==========================================================
// stddefs.h ---------------------------------------------------------
#ifndef __GAUGE3D_STDDEFS_H__INCLUDED__
#include "gauge3d/stddefs.h"
#endif //redundant include guard

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_RENDERER_LIGHT_H__INCLUDED__
#include "gauge3d/renderer/light.h"
#endif //redundant include guard

// local package -----------------------------------------------------

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpGLLight;
typedef GSmartPointer<GpGLLight> pGpGLLight;
class GpGLLight: public GLight
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x3978220e)
public:
	GpGLLight(pGpGLLight* link);
	virtual ~GpGLLight();
	
	void Register(int num); //do OpenGL calls to enable the light.
	                        //(num is the light number to use, 0-7)
	
	// operations ------------------------------------------------------
	//All lights start out hidden.  After you have cached the light, call Show() to
	// enable it.  Show and Hide are a good way to turn lights on and off.
	virtual void Show();
	virtual void Hide();
	//Call Remove to permenantly remove the light from the scene.
	virtual void Remove();
	
	// access ----------------------------------------------------------
	//The the location of the light.
	virtual void Position(const GVector& position);
	
	//Instead of an origin, you can specify a direction for an infinite light source.
	virtual void Direction(const GVector& direction);
	
	//Use these to set the color of the light.
	virtual void Color(float red, float green, float blue);
	virtual void Color(const float color[]);   //3-element array: RGB
	
	//LightProps sets the light properties.
	virtual void LightProps(float ambient, float diffuse, float specular = 0.0);
	virtual void LightProps2v(const float lightProps[]);  //ambient and diffuse
	virtual void LightProps3v(const float lightProps[]);  //ambient, diffuse, and specular
	
	//Attenuation allows you to set linear and quadratic attenuation factors.
	//(this makes the light get dimmer as you get further away)
	virtual void Attenuation(float linear, float quadratic = 0.0);
	
	//Spotlight makes the light into a spotlight pointing in the specified
	// direction and having the specified width (angle in degrees).  exponent
	// specifies how focused the spotlight is.  To disable, specify -1.0 for width.
	virtual void Spotlight(const GVector& direction, float exponent, float width);
	
	
	inline pGpGLLight Next() {return mNext;}
	
private:
	bool mEnabled;
	
	GVector mPosition;
	bool mDirectional;      //if the light is directional, this is true and mOrigin
	                        // is the direction.
	float mColor[3];
	float mLightProps[3];
	float mAttenuation[2];
	
	GVector mSpotDirection;
	float mSpotExponent;
	float mSpotWidth;       //angle in degrees
	
	pGpGLLight  mNext;
	pGpGLLight* mPrev;      //actually a pointer to the previous' mNext field
};

} // namespace Plugins
} // namespace GAUGE3D

#endif //included
