/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: glplugin.cpp
 *      Package: OpenGL Renderer Plugin
 *    Component: Plugin
 *  Description: The actual plugin object and stuff
 * Dependencies: GLRenderer
 */

// includes ==========================================================
// GAUGE^3D ----------------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_PLUGIN_H__INCLUDED__
#include "gauge3d/plugins/plugin.h"
#endif

// local package -----------------------------------------------------
#include "glrenderer.h"

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class definition ==================================================
class GpGLPlugin: public GPlugin
{
	GAUGE3D_SET_UID(0x00000048, 0x54846eca, 0x397822e5)
public:
	// lifecycle -------------------------------------------------------
	GpGLPlugin() {};
	virtual ~GpGLPlugin() {};
	
	// inquiry ---------------------------------------------------------
	//these return the human-readable name and description of the plugin.
	virtual GString Name() {return "OpenGL Renderer";}
	virtual GString Description()
		{return "Uses OpenGL to display 3D images through your graphics accellerator";}
	
	//InUse returns true if the plugin is being used.  Otherwise, false.
	// The object loader uses this to determine when it is safe to unload
	// a plugin from RAM.
	virtual bool InUse() {return true;}  //FIXME
	
	// operations ------------------------------------------------------
	virtual pGObject CreateObject(const GString& className)
	{
		if(className == "GpGLRenderer")
			return new GpGLRenderer;
		return NULL;   //err...  we don't serve those
	}
	
	//When loading a file, if the plugin finds that it can't load the thing,
	// it should return NULL.
	virtual pGObject CreateObject(const GString& className, pGFile file)
	{
		return NULL;   //we don't serve any data file objects.
	}
	
	//The plugin tells the object loader what objects are available from
	// it by giving it an array of ObjectInfo objects.  Each one gives
	// info for one of the available objects.
	//ObjectList() returns ObjectInfos for every object available from
	// this plugin.
	virtual GArray<ObjectInfo> ObjectList()
	{
		GArray<ObjectInfo> info(1);
		info[0].mClassname   = "GpGLRenderer";
		info[0].mName        = "OpenGL Renderer";
		info[0].mDescription = "Uses OpenGL to display 3D images through your graphics accellerator";
		
		info[0].mBaseClasses    = GArray<GString>(1);
		info[0].mBaseClasses[0] = "GRenderer";
		info[0].mKeywords       = GArray<GString>(1);
		info[0].mKeywords[0]    = "opengl";
		
		return info;
	}
};

GAUGE3D_DECLARE_INTERFACES(GpGLPlugin)
	GAUGE3D_DECLARE_BASECLASS(GPlugin)
GAUGE3D_END_INTERFACES

GAUGE3D_EXPORT_PLUGIN(GpGLPlugin)

} //namespace Plugins
} //namespace GAUGE3D
