/* GAUGE^3D - Grand Unified (3D) Game Engine
 * Copyright (C) 2000 Kenton Varda
 * <http://www.gauge3d.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/*
 *     Filename: glscene.cpp
 *      Package: OpenGL Rendering Plugin
 *    Component: Scene
 *  Description: Basically just a list of sprites and lights
 * Dependencies: sprites, lights, etc.
 */

// includes ==========================================================
// component header --------------------------------------------------
#include "glscene.h"

// system ------------------------------------------------------------
// GAUGE^3D ----------------------------------------------------------
// local package -----------------------------------------------------
#ifndef __GAUGE3D_PLUGINS_RENDERER_OPENGL_GLCAMERA_H__INCLUDED__
#include "glcamera.h"
#endif

// begin namespace ===================================================
namespace GAUGE3D {
namespace Plugins {

// class member functions ============================================
// lifecycle ---------------------------------------------------------
GpGLScene::GpGLScene()
{
}
GpGLScene::~GpGLScene()
{
}

// operations --------------------------------------------------------
//Add lights and sprites to the scene.
pGSprite
GpGLScene::NewSprite()
{
	return new GpGLSprite(&mSpriteList);
}

pGLight
GpGLScene::NewLight()
{
	return new GpGLLight(&mLightList);
}

pGCamera
GpGLScene::NewCamera()
{
	return new GpGLCamera(this);
}

//Render the scene, assuming the projection and modelview matrices have
// already been set up so that the scene can be rendered in world coordinates.
void
GpGLScene::Draw(GVector viewpoint, GQuaternion angle, GFrustum viewFrustum, float geoLod)
{
	pGpGLLight light = mLightList;
	int i = 0;
	while(light != NULL)
	{
		light->Register(i);
		light = light->Next();
		i++;
	}
	while(i < 8)
	{
		glDisable((GLenum)(GL_LIGHT0 + i));
		i++;
	}
	
	pGpGLSprite sprite = mSpriteList;
	while(sprite != NULL)
	{
		if(sprite->HasAlpha())
		{
			//Sprite has alpha -- it should be in the alpha list.
			pGpGLSprite nextSprite = sprite->Next();
			
			//remove from current list.
			sprite->Remove();
			//add to alpha list
			sprite->AddToList(&mAlphaSpriteList);
			
			//process the next sprite
			sprite = nextSprite;
		}
		else
		{
			//Sprite has no alpha.  Draw it.
			sprite->Draw(viewpoint, angle, viewFrustum, geoLod);
			
			sprite = sprite->Next();
		}
	}
	
	sprite = mAlphaSpriteList;
	while(sprite != NULL)
	{
		sprite->Draw(viewpoint, angle, viewFrustum, geoLod);
		
		if(!sprite->HasAlpha())
		{
			//Sprite does not have alpha anymore -- move it back to the main list.
			pGpGLSprite nextSprite = sprite->Next();
			
			//remove from current list.
			sprite->Remove();
			//add to main sprite list
			sprite->AddToList(&mSpriteList);
			
			//process next sprite
			sprite = nextSprite;
		}
		else
			sprite = sprite->Next();
	}
}


GAUGE3D_DECLARE_INTERFACES(GpGLScene)
	GAUGE3D_DECLARE_BASECLASS(GScene)
GAUGE3D_END_INTERFACES

} //namespace Plugins
} //namespace GAUGE3D
